@@ -172,12 +172,8 @@ static void i915LayoutTextureImages( i915ContextPtr i915,
 	 
 	 t->intel.image[0][i].offset = total_height * pitch;
 	 t->intel.image[0][i].internalFormat = baseImage->_BaseFormat;
-	 if (t->intel.image[0][i].image->IsCompressed)
-	 {
-	   if (t->intel.image[0][i].image->Height > 4)
-	     total_height += t->intel.image[0][i].image->Height/4;
-	   else
-	     total_height += 1;
+	 if (t->intel.image[0][i].image->IsCompressed) {
+	    total_height += (t->intel.image[0][i].image->Height + 3) / 4;
 	 }
 	 else
 	   total_height += MAX2(2, t->intel.image[0][i].image->Height);
@@ -456,9 +452,14 @@ static void i915SetTexImages( i915ContextPtr i915,
      textureFormat = (MAPSURF_COMPRESSED | MT_COMPRESS_FXT1);
      break;
 
-   case MESA_FORMAT_DEPTH_COMPONENT16:
+   case MESA_FORMAT_Z16:
       t->intel.texelBytes = 2;
-      textureFormat = (MAPSURF_16BIT | MT_16BIT_L16);
+      if (tObj->DepthMode == GL_ALPHA)
+	  textureFormat = (MAPSURF_16BIT | MT_16BIT_A16);
+      else if (tObj->DepthMode == GL_INTENSITY)
+	  textureFormat = (MAPSURF_16BIT | MT_16BIT_I16);
+      else
+	  textureFormat = (MAPSURF_16BIT | MT_16BIT_L16);
       break;
 
    case MESA_FORMAT_RGBA_DXT1:
@@ -483,7 +484,7 @@ static void i915SetTexImages( i915ContextPtr i915,
      break;
 
 #if 0
-   case MESA_FORMAT_DEPTH_COMPONENT_X8Z24:
+   case MESA_FORMAT_Z24_S8:
       t->intel.texelBytes = 4;
       textureFormat = (MAPSURF_32BIT | MT_32BIT_xL824);
       break;
@@ -495,12 +496,19 @@ static void i915SetTexImages( i915ContextPtr i915,
       abort();
    }
 
-
-   if (i915->intel.intelScreen->deviceID == PCI_CHIP_I945_G ||
-       i915->intel.intelScreen->deviceID == PCI_CHIP_I945_GM)
-      i945LayoutTextureImages( i915, tObj );	 
-   else
-      i915LayoutTextureImages( i915, tObj );
+   switch (i915->intel.intelScreen->deviceID) {
+   case PCI_CHIP_I945_G:
+   case PCI_CHIP_I945_GM:
+   case PCI_CHIP_I945_GME:
+   case PCI_CHIP_G33_G:
+   case PCI_CHIP_Q33_G:
+   case PCI_CHIP_Q35_G:
+       i945LayoutTextureImages( i915, tObj );
+       break;
+   default:
+       i915LayoutTextureImages( i915, tObj );
+       break;
+   }
 
    t->Setup[I915_TEXREG_MS3] = 
       (((tObj->Image[0][t->intel.base.firstLevel]->Height - 1) << MS3_HEIGHT_SHIFT) |
@@ -730,10 +738,16 @@ static GLboolean enable_tex_common( GLcontext *ctx, GLuint unit )
    }
 
    /* Fallback if there's a texture border */
-   if ( tObj->Image[0][tObj->BaseLevel]->Border > 0 ) {
+   if ( tObj->Image[0][tObj->BaseLevel]->Border > 0 ||
+        ((tObj->Image[0][tObj->BaseLevel]->_BaseFormat == GL_DEPTH_COMPONENT) &&
+         ((tObj->WrapS == GL_CLAMP_TO_BORDER) ||
+          (tObj->WrapT == GL_CLAMP_TO_BORDER)))) {
       return GL_FALSE;
    }
 
+   if (tObj->Target == GL_TEXTURE_1D &&
+       tObj->CompareMode == GL_COMPARE_R_TO_TEXTURE_ARB)
+      return GL_FALSE;
 
    /* Update state if this is a different texture object to last
     * time.
