@@ -49,7 +49,7 @@
 
 
 /* no borders! can't halve 1x1! (stride > width * comp) not allowed */
-void
+static void
 _mesa_halve2x2_teximage2d ( GLcontext *ctx,
 			    struct gl_texture_image *texImage,
 			    GLuint bytesPerPixel,
@@ -66,7 +66,7 @@ _mesa_halve2x2_teximage2d ( GLcontext *ctx,
    GLuint bpt = 0;
    GLubyte *_s = NULL;
    GLubyte *_d = NULL;
-   GLenum _t;
+   GLenum _t = 0;
 
    if (texImage->TexFormat->MesaFormat == MESA_FORMAT_RGB565) {
       _t = GL_UNSIGNED_SHORT_5_6_5_REV;
@@ -95,7 +95,8 @@ _mesa_halve2x2_teximage2d ( GLcontext *ctx,
                               srcRowStride, /* dstRowStride */
                               0, /* dstImageStride */
                               srcWidth, srcHeight, 1,
-                              texImage->Format, _t, srcImage, &ctx->DefaultPacking);
+                              texImage->_BaseFormat, _t,
+                              srcImage, &ctx->DefaultPacking);
    }
 
    if (srcHeight == 1) {
@@ -136,7 +137,7 @@ _mesa_halve2x2_teximage2d ( GLcontext *ctx,
    if (bpt) {
       src = _s;
       dst = _d;
-      texImage->TexFormat->StoreImage(ctx, 2, texImage->Format,
+      texImage->TexFormat->StoreImage(ctx, 2, texImage->_BaseFormat,
                                       texImage->TexFormat, dstImage,
                                       0, 0, 0, /* dstX/Y/Zoffset */
                                       dstWidth * bpt,
@@ -322,14 +323,11 @@ fxDDTexParam(GLcontext * ctx, GLenum target, struct gl_texture_object *tObj,
 	 ti->LODblend = FXFALSE;
 	 break;
       case GL_NEAREST_MIPMAP_LINEAR:
-         /* [koolsmoky]
-          * trilinear is bugged! mipmap blending produce
-          * incorrect filtered colors for the smallest mipmap levels.
-          * [dBorca]
+         /* [dBorca]
           * currently Napalm can't do single-pass trilinear,
           * because the way its combiners are set. So we fall back
           * to GL_NEAREST_MIPMAP_NEAREST. We'll let true trilinear
-          * enabled for V2, V3. If user shoots foot, not our problem!
+          * enabled for V2, V3.
           */
          if (!fxMesa->HaveCmbExt) {
 	    if (fxMesa->haveTwoTMUs) {
@@ -348,14 +346,11 @@ fxDDTexParam(GLcontext * ctx, GLenum target, struct gl_texture_object *tObj,
 	 ti->LODblend = FXFALSE;
 	 break;
       case GL_LINEAR_MIPMAP_LINEAR:
-         /* [koolsmoky]
-          * trilinear is bugged! mipmap blending produce
-          * incorrect filtered colors for the smallest mipmap levels.
-          * [dBorca]
+         /* [dBorca]
           * currently Napalm can't do single-pass trilinear,
           * because the way its combiners are set. So we fall back
           * to GL_LINEAR_MIPMAP_NEAREST. We'll let true trilinear
-          * enabled for V2, V3. If user shoots foot, not our problem!
+          * enabled for V2, V3.
           */
          if (!fxMesa->HaveCmbExt) {
             if (fxMesa->haveTwoTMUs) {
@@ -509,14 +504,14 @@ fxDDIsTextureResident(GLcontext *ctx, struct gl_texture_object *tObj)
 static GrTexTable_t
 convertPalette(const fxMesaContext fxMesa, FxU32 data[256], const struct gl_color_table *table)
 {
-   const GLubyte *tableUB = (const GLubyte *) table->Table;
+   const GLubyte *tableUB = (const GLubyte *) table->TableUB;
    GLint width = table->Size;
    FxU32 r, g, b, a;
    GLint i;
 
    ASSERT(table->Type == GL_UNSIGNED_BYTE);
 
-   switch (table->Format) {
+   switch (table->_BaseFormat) {
    case GL_INTENSITY:
       for (i = 0; i < width; i++) {
 	 r = tableUB[i];
@@ -585,8 +580,8 @@ fxDDTexPalette(GLcontext * ctx, struct gl_texture_object *tObj)
 		 tObj->Name, (GLuint) tObj->DriverData);
       }
       /* This might be a proxy texture. */
-      if (!tObj->Palette.Table)
-         return; 
+      if (!tObj->Palette.TableUB) /* TODO: is that ok or should it be TableF? */
+         return;
       if (!tObj->DriverData)
          tObj->DriverData = fxAllocTexObjData(fxMesa);
       ti = fxTMGetTexInfo(tObj);
@@ -1000,6 +995,7 @@ fetch_rgba_dxt5(const struct gl_texture_image *texImage,
 }
 
 
+#if 0 /* break glass in case of emergency */
 static void
 PrintTexture(int w, int h, int c, const GLubyte * data)
 {
@@ -1015,6 +1011,7 @@ PrintTexture(int w, int h, int c, const GLubyte * data)
       fprintf(stderr, "\n");
    }
 }
+#endif
 
 
 const struct gl_texture_format *
@@ -1030,11 +1027,6 @@ fxDDChooseTextureFormat( GLcontext *ctx, GLint internalFormat,
 
    switch (internalFormat) {
    case GL_COMPRESSED_RGB:
-#if FX_TC_NCC
-      if (fxMesa->HaveTexus2) {
-         return &_mesa_texformat_argb8888;
-      }
-#endif
      /* intentional fall through */
    case 3:
    case GL_RGB:
@@ -1052,11 +1044,6 @@ fxDDChooseTextureFormat( GLcontext *ctx, GLint internalFormat,
    case GL_RGBA4:
       return &_mesa_texformat_argb4444;
    case GL_COMPRESSED_RGBA:
-#if FX_TC_NCC
-      if (fxMesa->HaveTexus2) {
-         return &_mesa_texformat_argb8888;
-      }
-#endif
      /* intentional fall through */
    case 4:
    case GL_RGBA:
@@ -1252,7 +1239,7 @@ adjust2DRatio (GLcontext *ctx,
          return GL_FALSE;
       }
 
-      texImage->TexFormat->StoreImage(ctx, 2, texImage->Format,
+      texImage->TexFormat->StoreImage(ctx, 2, texImage->_BaseFormat,
                                       texImage->TexFormat, tempImage,
                                       0, 0, 0, /* dstX/Y/Zoffset */
                                       width * texelBytes, /* dstRowStride */
@@ -1267,6 +1254,7 @@ adjust2DRatio (GLcontext *ctx,
             + xoffset * mml->wScale) * texelBytes;
 
       _mesa_rescale_teximage2d(texelBytes,
+                               width,
                                dstRowStride, /* dst stride */
                                width, height,
                                newWidth, newHeight,
@@ -1290,11 +1278,12 @@ adjust2DRatio (GLcontext *ctx,
                               width, height, 1,
                               format, type, pixels, packing);
       _mesa_rescale_teximage2d(rawBytes,
+                               width,
                                newWidth * rawBytes, /* dst stride */
                                width, height, /* src */
                                newWidth, newHeight, /* dst */
                                rawImage /*src*/, tempImage /*dst*/ );
-      texImage->TexFormat->StoreImage(ctx, 2, texImage->Format,
+      texImage->TexFormat->StoreImage(ctx, 2, texImage->_BaseFormat,
                                       texImage->TexFormat, texImage->Data,
                                       xoffset * mml->wScale, yoffset * mml->hScale, 0, /* dstX/Y/Zoffset */
                                       dstRowStride,
@@ -1325,7 +1314,7 @@ fxDDTexImage2D(GLcontext * ctx, GLenum target, GLint level,
 
    if (TDFX_DEBUG & VERBOSE_TEXTURE) {
        fprintf(stderr, "fxDDTexImage2D: id=%d int 0x%x  format 0x%x  type 0x%x  %dx%d\n",
-                       texObj->Name, texImage->IntFormat, format, type,
+                       texObj->Name, texImage->InternalFormat, format, type,
                        texImage->Width, texImage->Height);
    }
 
@@ -1360,7 +1349,7 @@ fxDDTexImage2D(GLcontext * ctx, GLenum target, GLint level,
 
 #if FX_COMPRESS_S3TC_AS_FXT1_HACK
    /* [koolsmoky] substitute FXT1 for DXTn and Legacy S3TC */
-   if (texImage->IsCompressed) {
+   if (!ctx->Mesa_DXTn && texImage->IsCompressed) {
      switch (internalFormat) {
      case GL_COMPRESSED_RGB_S3TC_DXT1_EXT:
      case GL_RGB_S3TC:
@@ -1374,7 +1363,7 @@ fxDDTexImage2D(GLcontext * ctx, GLenum target, GLint level,
      case GL_RGBA4_S3TC:
        internalFormat = GL_COMPRESSED_RGBA_FXT1_3DFX;
      }
-     texImage->IntFormat = internalFormat;
+     texImage->InternalFormat = internalFormat;
    }
 #endif
 #if FX_TC_NAPALM
@@ -1386,7 +1375,7 @@ fxDDTexImage2D(GLcontext * ctx, GLenum target, GLint level,
          texNapalm = GL_COMPRESSED_RGBA_FXT1_3DFX;
       }
       if (texNapalm) {
-         texImage->IntFormat = internalFormat = texNapalm;
+         texImage->InternalFormat = internalFormat = texNapalm;
          texImage->IsCompressed = GL_TRUE;
       }
    }
@@ -1411,10 +1400,10 @@ fxDDTexImage2D(GLcontext * ctx, GLenum target, GLint level,
                                                                1,
                                                                internalFormat);
       dstRowStride = _mesa_compressed_row_stride(internalFormat, mml->width);
-      texImage->Data = MESA_PBUFFER_ALLOC(texImage->CompressedSize);
+      texImage->Data = _mesa_malloc(texImage->CompressedSize);
    } else {
       dstRowStride = mml->width * texelBytes;
-      texImage->Data = MESA_PBUFFER_ALLOC(mml->width * mml->height * texelBytes);
+      texImage->Data = _mesa_malloc(mml->width * mml->height * texelBytes);
    }
    if (!texImage->Data) {
       _mesa_error(ctx, GL_OUT_OF_MEMORY, "glTexImage2D");
@@ -1441,7 +1430,7 @@ fxDDTexImage2D(GLcontext * ctx, GLenum target, GLint level,
       else {
          /* no rescaling needed */
          /* unpack image, apply transfer ops and store in texImage->Data */
-         texImage->TexFormat->StoreImage(ctx, 2, texImage->Format,
+         texImage->TexFormat->StoreImage(ctx, 2, texImage->_BaseFormat,
                                          texImage->TexFormat, texImage->Data,
                                          0, 0, 0, /* dstX/Y/Zoffset */
                                          dstRowStride,
@@ -1449,39 +1438,7 @@ fxDDTexImage2D(GLcontext * ctx, GLenum target, GLint level,
                                          width, height, 1,
                                          format, type, pixels, packing);
       }
-#if FX_TC_NCC
-      if (fxMesa->HaveTexus2) {
-         GLenum texNCC = 0;
-         GLuint texSize = mml->width * mml->height;
-         if (internalFormat == GL_COMPRESSED_RGB) {
-            texNCC = GR_TEXFMT_YIQ_422;
-         } else if (internalFormat == GL_COMPRESSED_RGBA) {
-            texNCC = GR_TEXFMT_AYIQ_8422;
-            texSize <<= 1;
-         }
-         if (texNCC) {
-            TxMip txMip, pxMip;
-            GLubyte *tempImage = MESA_PBUFFER_ALLOC(texSize);
-            if (!tempImage) {
-               _mesa_error(ctx, GL_OUT_OF_MEMORY, "glTexImage2D");
-               return;
-            }
-            txMip.width = mml->width;
-            txMip.height = mml->height;
-            txMip.depth = 1;
-            txMip.data[0] = texImage->Data;
-            pxMip.data[0] = tempImage;
-            fxMesa->Glide.txMipQuantize(&pxMip, &txMip, texNCC, TX_DITHER_ERR, TX_COMPRESSION_HEURISTIC);
-            if (level == 0) {
-               fxMesa->Glide.txPalToNcc((GuNccTable *)(&(ti->palette)), pxMip.pal);
-            }
-            MESA_PBUFFER_FREE(texImage->Data);
-            texImage->Data = tempImage;
-            mml->glideFormat = texNCC;
-         }
-      }
-#endif
-   
+
       /* GL_SGIS_generate_mipmap */
       if (level == texObj->BaseLevel && texObj->GenerateMipmap) {
          GLint mipWidth, mipHeight;
@@ -1489,9 +1446,9 @@ fxDDTexImage2D(GLcontext * ctx, GLenum target, GLint level,
          struct gl_texture_image *mipImage;
          const struct gl_texture_unit *texUnit = &ctx->Texture.Unit[ctx->Texture.CurrentUnit];
          const GLint maxLevels = _mesa_max_texture_levels(ctx, texObj->Target);
-   
+
          assert(!texImage->IsCompressed);
-   
+
          while (level < texObj->MaxLevel && level < maxLevels - 1) {
             mipWidth = width / 2;
             if (!mipWidth) {
@@ -1544,10 +1501,6 @@ fxDDTexSubImage2D(GLcontext * ctx, GLenum target, GLint level,
    tfxMipMapLevel *mml;
    GLint texelBytes, dstRowStride;
 
-   /* [dBorca] Hack alert:
-    * FX_TC_NCC not supported
-    */
-
    if (TDFX_DEBUG & VERBOSE_TEXTURE) {
        fprintf(stderr, "fxDDTexSubImage2D: id=%d\n", texObj->Name);
    }
@@ -1563,11 +1516,11 @@ fxDDTexSubImage2D(GLcontext * ctx, GLenum target, GLint level,
    assert(mml);
 
    assert(texImage->Data);	/* must have an existing texture image! */
-   assert(texImage->Format);
+   assert(texImage->_BaseFormat);
 
    texelBytes = texImage->TexFormat->TexelBytes;
    if (texImage->IsCompressed) {
-      dstRowStride = _mesa_compressed_row_stride(texImage->IntFormat, mml->width);
+      dstRowStride = _mesa_compressed_row_stride(texImage->InternalFormat, mml->width);
    } else {
       dstRowStride = mml->width * texelBytes;
    }
@@ -1590,7 +1543,7 @@ fxDDTexSubImage2D(GLcontext * ctx, GLenum target, GLint level,
    }
    else {
       /* no rescaling needed */
-      texImage->TexFormat->StoreImage(ctx, 2, texImage->Format,
+      texImage->TexFormat->StoreImage(ctx, 2, texImage->_BaseFormat,
                                       texImage->TexFormat, (GLubyte *) texImage->Data,
                                       xoffset, yoffset, 0, /* dstX/Y/Zoffset */
                                       dstRowStride,
@@ -1713,7 +1666,7 @@ fxDDCompressedTexImage2D (GLcontext *ctx, GLenum target,
                                                                mml->height,
                                                                1,
                                                                internalFormat);
-      texImage->Data = MESA_PBUFFER_ALLOC(texImage->CompressedSize);
+      texImage->Data = _mesa_malloc(texImage->CompressedSize);
       if (!texImage->Data) {
          _mesa_error(ctx, GL_OUT_OF_MEMORY, "glCompressedTexImage2D");
          return;
@@ -1787,12 +1740,12 @@ fxDDCompressedTexSubImage2D( GLcontext *ctx, GLenum target,
    mml = FX_MIPMAP_DATA(texImage);
    assert(mml);
 
-   srcRowStride = _mesa_compressed_row_stride(texImage->IntFormat, width);
+   srcRowStride = _mesa_compressed_row_stride(texImage->InternalFormat, width);
 
-   destRowStride = _mesa_compressed_row_stride(texImage->IntFormat,
+   destRowStride = _mesa_compressed_row_stride(texImage->InternalFormat,
                                                mml->width);
    dest = _mesa_compressed_image_address(xoffset, yoffset, 0,
-                                         texImage->IntFormat,
+                                         texImage->InternalFormat,
                                          mml->width,
                               (GLubyte*) texImage->Data);
 
@@ -1808,9 +1761,9 @@ fxDDCompressedTexSubImage2D( GLcontext *ctx, GLenum target,
     * see fxDDCompressedTexImage2D for caveats
     */
    if (mml->wScale != 1 || mml->hScale != 1) {
-      srcRowStride = _mesa_compressed_row_stride(texImage->IntFormat, texImage->Width);
+      srcRowStride = _mesa_compressed_row_stride(texImage->InternalFormat, texImage->Width);
 
-      destRowStride = _mesa_compressed_row_stride(texImage->IntFormat,
+      destRowStride = _mesa_compressed_row_stride(texImage->InternalFormat,
                                                mml->width);
       _mesa_upscale_teximage2d(srcRowStride, texImage->Height / 4,
                                destRowStride, mml->height / 4,
