@@ -1618,7 +1618,8 @@ parse_init_declarator(slang_parse_ctx * C, slang_output_ctx * O,
       A.program = O->program;
       A.vartable = O->vartable;
       A.curFuncEndLabel = NULL;
-      _slang_codegen_global_variable(&A, var, C->type);
+      if (!_slang_codegen_global_variable(&A, var, C->type))
+         return 0;
    }
 
    /* allocate global address space for a variable with a known size */
@@ -2123,6 +2124,9 @@ _slang_compile(GLcontext *ctx, struct gl_shader *shader)
       type = SLANG_UNIT_FRAGMENT_SHADER;
    }
 
+   if (!shader->Source)
+      return GL_FALSE;
+
    ctx->Shader.MemPool = _slang_new_mempool(1024*1024);
 
    /* XXX temporary hack */
@@ -2134,7 +2138,7 @@ _slang_compile(GLcontext *ctx, struct gl_shader *shader)
          progTarget = GL_FRAGMENT_PROGRAM_ARB;
       shader->Programs
          = (struct gl_program **) malloc(sizeof(struct gl_program*));
-      shader->Programs[0] = _mesa_new_program(ctx, progTarget, 1);
+      shader->Programs[0] = ctx->Driver.NewProgram(ctx, progTarget, 1);
       shader->NumPrograms = 1;
 
       shader->Programs[0]->Parameters = _mesa_new_parameter_list();
